/********************************************************************************************
 * AI Game Development: Learning & Reactive Behaviors
 *
 * Copyright (c) 2002-2003 Alex J. Champandard, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under the terms 
 * of the GNU General Public License (GPL) as published by the Free Software Foundation.  
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY, 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the attached LICENSE file for details.
 *
 * 
 * File:         Jwha3BrainBase.h
 * Description:      Definition of the Brain class to be customised by the user.
 * Author:         Alex J. Champandard
 * Modified:        John W. Hamill - Modified from Spin, Brutus, ...
 * Contact:         alex@ai-depot.com
 * 
 ********************************************************************************************/

#pragma once


// ------------------------------------------------------------------------------------------
// Essential Inclusions
// ------------------------------------------------------------------------------------------
#include "Base.h"
#include <fear/architecture.hpp>
#include "../../common/Combat.h" // Prototype for FindNearestPlayer()


namespace fear {



// ------------------------------------------------------------------------------------------
// Forward Declarations
// ------------------------------------------------------------------------------------------
class Entity;



// ==========================================================================================
// Jwha3Brain
// ------------------------------------------------------------------------------------------
/// This brain class contains the AI for the animat.  The user implements this class, adding 
/// new methods and member variables as necessary.  The base class is generated automatically
/// by the tools from the definition, and contains functions and variables to simplify the 
/// code.  The brain is also an AI architecture as it relies on nested components.
// ==========================================================================================
class Jwha3Brain : public Jwha3BrainBase, public Architecture
{

// ------------------------------------------------------------------------------------------
// Common Interface
// ------------------------------------------------------------------------------------------

public:

   /**
    * Main entry point for the AI called regularly by the framework.
    */
   void Think();


// ------------------------------------------------------------------------------------------
// Private Implementation
// ------------------------------------------------------------------------------------------

private:

// ------------------------------------------------------------------------------------------
// Common Routines
// ------------------------------------------------------------------------------------------

    template <class T> friend bool FindNearestPlayer(T& self, Vec3f& position, Entity** ent);


// ------------------------------------------------------------------------------------------
// Life-Cycle Management
// ------------------------------------------------------------------------------------------

protected:

   /**
    * Constructor called by the framework to create a new brain, used for memory allocation
    * and trivial initialisation of variables.
    */
   Jwha3Brain();

   /**
    * Destructor frees memory when requested by the framework.
    */
   ~Jwha3Brain();

// ------------------------------------------------------------------------------------------
// Member Variables
// ------------------------------------------------------------------------------------------

private:

   Vec3f mxvPlayer; ///< remember the relative position of Human player

   Vec3f mxvAway1;  // Remember 180 deg. & 46 units away from player
   Vec3f mxvAway2;  // Remember 180 deg. & 50 units away from player

   float mfDegToTurn; // Deg. to turn away from Human Player
   bool  mbTurnCCW;  // true - Turn CCW, false - Turn CW

   float mfTurn;   ///< the yaw used to request angular turns if a collision is detected


// ------------------------------------------------------------------------------------------
// Inherited Base Variables
// ------------------------------------------------------------------------------------------

//   Motion::iRelative* motion;
//   Physics::iRelative* physics;
//   Space::iSpace* space;
//   Vision::iSymbolic* vision;
//   Debug::iVisual* debug;


}; // class Jwha3Brain



} // namespace fear

